@@ -47,17 +47,16 @@ using namespace std;
 #define ADD_RO_PROPERTY(NAME) \
   add_property(#NAME, &cl::NAME)
 
+#define CHECK_VERSION(MAJOR, MINOR, PATCH) \
+  (MAJOR << 16) + \
+  (MINOR << 8) + \
+  (PATCH << 0)
 
+#define TAGLIB_HEX_VERSION CHECK_VERSION(TAGLIB_MAJOR_VERSION, TAGLIB_MINOR_VERSION, TAGLIB_PATCH_VERSION)
 
-
-#define TAGPY_TAGLIB_HEX_VERSION \
-  (TAGLIB_MAJOR_VERSION << 16) + \
-  (TAGLIB_MINOR_VERSION << 8) + \
-  (TAGLIB_PATCH_VERSION << 0)
-
-#if (TAGPY_TAGLIB_HEX_VERSION < 0x10400)
+#if CHECK_VERSION(1,9,0) < TAGLIB_HEX_VERSION
 #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-#warning TagPy is meant to wrap TagLib 1.4 and above.
+#warning TagPy is meant to wrap TagLib 1.9 and above.
 #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 #endif
 
@@ -129,7 +128,7 @@ namespace {
   // List
   // -------------------------------------------------------------
   template<typename Value>
-  Value &List_getitem(List<Value> &l, TagLib::uint i)
+  Value &List_getitem(List<Value> &l, uint i)
   {
     if (i >= l.size())
     {
@@ -140,7 +139,7 @@ namespace {
   }
 
   template<typename Value>
-  void List_setitem(List<Value> &l, TagLib::uint i, Value v)
+  void List_setitem(List<Value> &l, uint i, Value v)
   {
     if (i >= l.size())
     {
@@ -177,7 +176,7 @@ namespace {
   // PointerList
   // -------------------------------------------------------------
   template<typename Value>
-  Value *&PointerList_getitem(List<Value *> &l, TagLib::uint i)
+  Value *&PointerList_getitem(List<Value *> &l, uint i)
   {
     if (i >= l.size())
     {
@@ -188,7 +187,7 @@ namespace {
   }
 
   template<typename Value>
-  void PointerList_setitem(List<Value *> &l, TagLib::uint i, auto_ptr<Value> v)
+  void PointerList_setitem(List<Value *> &l, uint i, auto_ptr<Value> v)
   {
     if (i >= l.size())
     {
@@ -221,3 +220,22 @@ namespace {
       ;
   }
 }
+
+template <class T>
+struct TagWrap : T, wrapper<T>
+{
+    String title() const { return this->get_override("title")(); }
+    String artist() const { return this->get_override("artist")(); }
+    String album() const { return this->get_override("album")(); }
+    String comment() const { return this->get_override("comment")(); }
+    String genre() const { return this->get_override("genre")(); }
+    unsigned int year() const { return this->get_override("year")(); }
+    unsigned int track() const { return this->get_override("track")(); }
+    void setTitle(const String &v) const { this->get_override("setTitle")(v); }
+    void setArtist(const String &v) const { this->get_override("setArtist")(v); }
+    void setAlbum(const String &v) const { this->get_override("setAlbum")(v); }
+    void setComment(const String &v) const { this->get_override("setComment")(v); }
+    void setGenre(const String &v) const { this->get_override("setGenre")(v); }
+    void setYear(unsigned int i) const { this->get_override("setYear")(i); }
+    void setTrack(unsigned int i) const { this->get_override("setTrack")(i); }
+};
